package #PACKAGE_NAME#;

import java.net.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import java.io.*;

/* BeanContainer used to contain a set of beans and display them graphically */

public class #CONTAINER_NAME# extends java.awt.Container implements Serializable
{
    private static boolean debug = false;

    public #CONTAINER_NAME#()
    {
        beans = new Vector();
        setLayout(null);
        Dimension d = getPreferredSize();
        setSize (d.width, d.height);
    }
    
    public synchronized int addBean(Object bean)
    {
        beans.addElement(bean);
        return beans.size()-1;
    }
    
    // bug in Container lightweight stuff. must override this to avoid cast exception
    public void validate() {
        if (!isValid() && getPeer() instanceof java.awt.peer.LightweightPeer)
            return;
        else super.validate();
    }

    public Dimension getPreferredSize() {
        int w = 0;
        int h = 0;
        int minx, miny, maxx, maxy;

        minx = miny = maxx = maxy = 0;

        for (int i = 0; i < beans.size(); i++){
            Object bean = beans.elementAt (i);
            if (Beans.isInstanceOf (bean, Component.class)){
                Component c = (Component)Beans.getInstanceOf(bean, Component.class);
                Rectangle bounds = c.getBounds();
                minx = (minx == 0) ? bounds.x : Math.min (minx, bounds.x);
                miny = (miny == 0) ? bounds.y : Math.min (miny, bounds.y);
                maxx = Math.max (maxx, bounds.x + bounds.width);                
                maxy = Math.max (maxy, bounds.y + bounds.height);
            }
        }

        w = minx + maxx;
        h = miny + maxy;

        debug("getPreferredSize: " + w + "," + h);
	return (new Dimension(w, h));
    }

    private void writeObject(ObjectOutputStream out) throws IOException,
        ClassNotFoundException
    {
        int n = beans.size();
        out.writeInt(n);
        for (int i = 0; i < n; i++) {
            out.writeObject(beans.elementAt(i));
        }
    }

    /**
     * Deserialize model data from input stream.
     */
    private void readObject(ObjectInputStream in) throws IOException,
        ClassNotFoundException
    {
        debug ("reading from  ObjectInputStream : " + in);
        //in.defaultReadObject();
        beans = new Vector();

        int n = in.readInt();
        for (int i = 0; i < n; i++) {
            Object bean = (Object) in.readObject();
            if (bean instanceof java.applet.Applet){
                initializeAppletBean (bean);
            }
            beans.addElement(bean);
        }
        setup();
    }


    private void setup()
    {
        //add bean components to this container
        Component c;

        for (int i = 0; i < beans.size(); i++){
            Object bean = beans.elementAt (i);
            if (Beans.isInstanceOf (bean, Component.class)){
                c = (Component)Beans.getInstanceOf(bean, Component.class);
                debug("adding component" + c);
                add (c);
            }
        }
    }


    private static void initializeAppletBean (Object applet)
    {
        if (Beans.isInstanceOf(applet, Applet.class)) {
                debug ("initializing applet bean");
                Applet apl = (Applet) Beans.getInstanceOf(applet, Applet.class);
                Class c = apl.getClass();
                ClassLoader cl = c.getClassLoader();

                //set applet stub so that applets can find their resources
                assignAppletStub (apl, cl, c);
                apl.init();
                apl.start();
        }
    }
    

    private static void assignAppletStub(Applet applet, ClassLoader cl, Class k) {
	String className = k.getName();
	String resourceName = className.replace('.','/').concat(".class");

	// This code is identical to portions of Beans.intantiate()
	URL objectUrl = null;
	URL codeBase = null;
	URL docBase = null;

	// Now get the URL correponding to the resource name.
	if (cl == null) {
	    objectUrl = ClassLoader.getSystemResource(resourceName);
	} else {
	    objectUrl = cl.getResource(resourceName);
	}

	// If we found a URL, we try to locate the docbase by taking
	// of the final path name component, and the code base by taking
	// of the complete resourceName.
	// So if we had a resourceName of "a/b/c.class" and we got an
	// objectURL of "file://bert/classes/a/b/c.class" then we would
	// want to set the codebase to "file://bert/classes/" and the
	// docbase to "file://bert/classes/a/b/"
	
	if (objectUrl != null) {
	    String s = objectUrl.toExternalForm();
	    if (s.endsWith(resourceName)) {
		try{ 
		    int ix = s.length() - resourceName.length();
		    codeBase = new URL(s.substring(0,ix));
		    docBase = codeBase;
		    ix = s.lastIndexOf('/');
		    if (ix >= 0) {
			docBase = new URL(s.substring(0,ix+1));
		    }
		} catch (Exception ex) {
		    docBase = codeBase = null;
		}
	    }
	}
	    	    
	// Setup a default context and stub.
	BeanBoxAppletContext context = new BeanBoxAppletContext(applet);
	BeanBoxAppletStub stub = new BeanBoxAppletStub(applet, context, codeBase, docBase);
	applet.setStub(stub);
	
	// object is always deserialized
	stub.active = true;
    }

    private static void debug(String msg) {
	if (debug) {
	    System.err.println("#CONTAINER_NAME#:: "+msg);
	}
    }


    private Vector beans;
}

    
//BeanBox applet support classes
class BeanBoxAppletContext implements AppletContext {
    private static boolean debug = false;

    Applet target;
    java.util.Hashtable imageCache = new java.util.Hashtable();

    BeanBoxAppletContext(Applet target) {
        this.target = target;
    }

    public AudioClip getAudioClip(URL url) {
	// We don't currently support audio clips in the Beans.instantiate
	// applet context, unless by some luck there exists a URL content
	// class that can generate an AudioClip from the audio URL.
	try {
	    return (AudioClip) url.getContent();
  	} catch (Exception ex) {
	    return null;
	}
    }

    public synchronized Image getImage(URL url) {
        debug ("getImage for url " + url);
        
	Object o = imageCache.get(url);
	if (o != null) {
	    return (Image)o;
	}
	try {
	    o = url.getContent();
	    if (o == null) {
		return null;
	    }
	    if (o instanceof Image) {
		imageCache.put(url, o);
		return (Image) o;
	    }
	    // Otherwise it must be an ImageProducer.
	    Image img = target.createImage((java.awt.image.ImageProducer)o);
	    imageCache.put(url, img);
	    return img;

  	} catch (Exception ex) {
	    return null;
	}
    }

    public Applet getApplet(String name) {
	return null;
    }

    public java.util.Enumeration getApplets() {
	java.util.Vector applets = new java.util.Vector();
	applets.addElement(target);
	return applets.elements();	
    }

    public void showDocument(URL url) {
	// We do nothing.
    }

    public void showDocument(URL url, String target) {
	// We do nothing.
    }

    public void showStatus(String status) {
	// We do nothing.
    }

    private static void debug(String msg) {
	if (debug) {
	    System.err.println("BeanBoxAppletContext:: "+msg);
	}
    }

}

/**
 * Package private support class.  This provides an AppletStub
 * for beans which are applets.
 */
class BeanBoxAppletStub implements AppletStub {
    transient boolean active;
    transient Applet target;
    transient AppletContext context;
    transient URL codeBase;
    transient URL docBase;

    BeanBoxAppletStub(Applet target,
		AppletContext context, URL codeBase, 
				URL docBase) {
        this.target = target;
	this.context = context;
	this.codeBase = codeBase;
	this.docBase = docBase;
    }

    public boolean isActive() {
	return active;
    }
    
    public URL getDocumentBase() {
	// use the root directory of the applet's class-loader
	return docBase;
    }

    public URL getCodeBase() {
	// use the directory where we found the class or serialized object.
	return codeBase;
    }

    public String getParameter(String name) {
	return null;
    }

    public AppletContext getAppletContext() {
	return context;
    }

    public void appletResize(int width, int height) {
	// we do nothing.
    }
}
