/******************************************************************
 * Copyright and Permission Notice                                *
 *                                                                *
 * Permission is granted to copy, use, modify, and merge this     *
 * software into your applications and to permit others to do any *
 * of the foregoing for non-commercial use only. You must include *
 * this permission and copyright notice in all copies and         *
 * modified versions of this software, and include attribution    *
 * to Taligent in all splash screens included in any application  *
 * using this software. You must require others to whom you       *
 * distribute this software to do likewise.                       *
 *                                                                *
 * To obtain an unrestricted commercial license to the software   *
 * source and/or binaries, please contact Taligent at             *
 * widgets@taligent.com                                           *
 *                                                                *
 * THE SOFTWARE IS PROVIDED IN ITS 'AS IS' CONDITION FOR          *
 * NON-COMMERCIAL USE ONLY. TALIGENT DISCLAIMS ANY LIABILITY OF   *
 * ANY KIND FOR DAMAGES WHATSOEVER RESULTING FROM THE USE OF      *
 * THIS SOFTWARE.                                                 *
 *                                                                *
 * (C) Copyright Taligent, Inc. 1996,1997-All Rights Reserved.    *
 * (C) Copyright IBM Corporation 1996,1997                        *
 ******************************************************************/

import java.awt.*;

import COM.taligent.widget.Paintable;
import COM.taligent.util.Comparator;

/**
 * Simple paintable icon for use in the MultiColumnListbox.
 * This icon also implements com.taligent.util.Comparator
 * so that it can be sorted.
 *
 * @author  Andy Clark, Taligent Inc.
 * @version 1.0
 */
public class StatusIcon
    extends Object
    implements Paintable, Comparator
    {
    // Data
    private Image image;
    private int   status;

    /**
     * Constructor.
     *
     * @param image     The image to be painted.
     * @param status    The status of the image. This value can
     *                  be used as the ID of the icon.
     */
    public StatusIcon(Image image, int status) {

        this.image  = image;
        this.status = status;
        }

    /** Returns the image. */
    public Image getImage()  {

        return image;
        }

    /** Returns the status of the image. */
    public int   getStatus() {

        return status;
        }

    // Methods for Paintable interface
    /**
     * Returns size needed to paint this icon. Not really
     * needed.
     */
    public Dimension getSize() {

        return new Dimension(20, 20);
        }

    /** Paints the icon. */
    public void paint(Graphics g) {

        g.drawImage(image, 0, 0, null);
        }

    // Method for Comparator interface
    /** Compares two status icons. */
    public int compare(Object a, Object b) {
        int aval = ((StatusIcon)a).getStatus();
        int bval = ((StatusIcon)b).getStatus();

        if (aval < bval)
            return -1;
        if (aval > bval)
            return 1;

        return 0;
        }
    }
